package be.iminds.ilabt.jfed.rspec_fx;

import be.iminds.ilabt.jfed.rspec_fx.model.controller.ExperimentBarrierSegment;
import be.iminds.ilabt.jfed.rspec_fx.model.controller.ExperimentCommand;
import be.iminds.ilabt.jfed.rspec_fx.model.controller.Timestamp;
import be.iminds.ilabt.jfed.rspec.request.ext.delay.Pipe;
import be.iminds.ilabt.jfed.rspec.request.ext.delay.SliverTypeShaping;

import javax.xml.stream.XMLEventFactory;
import javax.xml.stream.XMLEventWriter;
import javax.xml.stream.XMLStreamException;

import static be.iminds.ilabt.jfed.rspec.parser.RspecXmlConstants.*;

public class StaxRspecFxWriter {
    private static final StaxRspecFxWriter INSTANCE = new StaxRspecFxWriter();

    public static StaxRspecFxWriter getInstance() {
        return INSTANCE;
    }


    private XMLEventFactory eventFactory;


    private StaxRspecFxWriter() {
        eventFactory = XMLEventFactory.newInstance();
    }


    public void writeSliverTypeShaping(XMLEventWriter eventWriter, SliverTypeShaping sliverTypeShaping) throws XMLStreamException {

        eventWriter.add(eventFactory.createStartElement(Q_SLIVER_TYPE_SHAPING, null, null));

        for (Pipe pipe : sliverTypeShaping.getPipe()) {
            writePipe(eventWriter, pipe);
        }

        eventWriter.add(eventFactory.createEndElement(Q_SLIVER_TYPE_SHAPING, null));
    }

    private void writePipe(XMLEventWriter eventWriter, Pipe pipe) throws XMLStreamException {
        eventWriter.add(eventFactory.createStartElement(Q_PIPE, null, null));

        if (pipe.getSource() != null) {
            eventWriter.add(eventFactory.createAttribute(Q_ATT_SOURCE, pipe.getSource()));
        }
        if (pipe.getDest() != null) {
            eventWriter.add(eventFactory.createAttribute(Q_ATT_DEST, pipe.getDest()));
        }
        if (pipe.getCapacity() != null) {
            eventWriter.add(eventFactory.createAttribute(Q_ATT_CAPACITY, pipe.getCapacity().toString()));
        }
        if (pipe.getLatency() != null) {
            eventWriter.add(eventFactory.createAttribute(Q_ATT_LATENCY, pipe.getLatency().toString()));
        }
        if (pipe.getPacketLoss() != null) {
            eventWriter.add(eventFactory.createAttribute(Q_ATT_PACKET_LOSS, pipe.getPacketLoss().toString()));
        }
        if (pipe.isGentle() != null) {
            eventWriter.add(eventFactory.createAttribute(Q_ATT_GENTLE, pipe.isGentle().toString()));
        }
        if (pipe.isRed() != null) {
            eventWriter.add(eventFactory.createAttribute(Q_ATT_RED, pipe.isRed().toString()));
        }
        if (pipe.isQueueInBytes() != null) {
            eventWriter.add(eventFactory.createAttribute(Q_ATT_QUEUE_IN_BYTES, pipe.isQueueInBytes().toString()));
        }
        if (pipe.getLimit() != null) {
            eventWriter.add(eventFactory.createAttribute(Q_ATT_LIMIT, pipe.getLimit().toString()));
        }
        if (pipe.getMaxthresh() != null) {
            eventWriter.add(eventFactory.createAttribute(Q_ATT_MAXTRESH, pipe.getMaxthresh().toString()));
        }
        if (pipe.getThresh() != null) {
            eventWriter.add(eventFactory.createAttribute(Q_ATT_TRESH, pipe.getThresh().toString()));
        }
        if (pipe.getLinterm() != null) {
            eventWriter.add(eventFactory.createAttribute(Q_ATT_LINTERM, pipe.getLinterm().toString()));
        }
        if (pipe.getQWeight() != null) {
            eventWriter.add(eventFactory.createAttribute(Q_ATT_Q_WEIGHT, pipe.getQWeight().toString()));
        }

        eventWriter.add(eventFactory.createEndElement(Q_PIPE, null));

    }

    public void writeExperimentBarrierSegment(XMLEventWriter eventWriter, ExperimentBarrierSegment experimentBarrierSegment) throws XMLStreamException {
        eventWriter.add(eventFactory.createStartElement(Q_JFEDCOMMAND_EXP_BARRIER_SEG, null, null));

        eventWriter.add(eventFactory.createAttribute(Q_ATT_ORDER_NUMBER,
                Integer.toString(experimentBarrierSegment.getOrderNumber())));

        if (experimentBarrierSegment.getTag() != null) {
            eventWriter.add(eventFactory.createAttribute(Q_ATT_TAG, experimentBarrierSegment.getTag()));
        }

        eventWriter.add(eventFactory.createEndElement(Q_JFEDCOMMAND_EXP_BARRIER_SEG, null));
    }

    public void writeExperimentCommand(XMLEventWriter eventWriter, ExperimentCommand experimentCommand) throws XMLStreamException {
        eventWriter.add(eventFactory.createStartElement(Q_JFEDCOMMAND_EXP_COMMAND, null, null));

        eventWriter.add(eventFactory.createAttribute(Q_ATT_BARRIER_SEGMENT_ORDER_NUMBER,
                Integer.toString(experimentCommand.getBarrierSegmentOrderNumber())));

        if (experimentCommand.getTag() != null) {
            eventWriter.add(eventFactory.createAttribute(Q_ATT_TAG, experimentCommand.getTag()));
        }

        if(experimentCommand.getCommand() != null){
            eventWriter.add(eventFactory.createStartElement(Q_JFEDCOMMAND_COMMAND, null, null));
            eventWriter.add(eventFactory.createCharacters(experimentCommand.getCommand()));
            eventWriter.add(eventFactory.createEndElement(Q_JFEDCOMMAND_COMMAND, null));
        }

        if(experimentCommand.getDuration() != null){
            eventWriter.add(eventFactory.createStartElement(Q_JFEDCOMMAND_DURATION, null, null));
            writeTimestamp(eventWriter, experimentCommand.getDuration());
            eventWriter.add(eventFactory.createEndElement(Q_JFEDCOMMAND_DURATION, null));
        }

        if(experimentCommand.getStartingTime() != null){
            eventWriter.add(eventFactory.createStartElement(Q_JFEDCOMMAND_STARTING_TIME, null, null));
            writeTimestamp(eventWriter, experimentCommand.getStartingTime());
            eventWriter.add(eventFactory.createEndElement(Q_JFEDCOMMAND_STARTING_TIME, null));
        }


        eventWriter.add(eventFactory.createEndElement(Q_JFEDCOMMAND_EXP_COMMAND, null));
    }

    private void writeTimestamp(XMLEventWriter eventWriter, Timestamp timestamp) throws XMLStreamException {
        eventWriter.add(eventFactory.createStartElement(Q_JFEDCOMMAND_TIME, null, null));
        eventWriter.add(eventFactory.createCharacters(Double.toString(timestamp.getTime())));
        eventWriter.add(eventFactory.createEndElement(Q_JFEDCOMMAND_TIME, null));
    }


}
